@@ -3,7 +3,7 @@
  * Syscalls for uClibc
  *
  * Copyright (C) 2000 by Lineo, inc
- * Copyright (C) 2001 by Erik Andersen
+ * Copyright (C) 2001, 2002 by Erik Andersen
  * Written by Erik Andersen <andersen@codpoet.org>
  *
  * This program is free software; you can redistribute it and/or modify it
@@ -217,7 +217,7 @@ int setuid(uid_t uid)
 //#define __NR_getuid           24
 #ifdef L_getuid
 #include <unistd.h>
-_syscall0(gid_t, getuid);
+_syscall0(uid_t, getuid);
 #endif
 
 //#define __NR_stime            25
@@ -566,18 +566,61 @@ _syscall2(int, sethostname, const char *, name, size_t, len);
 #endif
 
 //#define __NR_setrlimit        75
+#ifndef __NR_ugetrlimit
+/* Only wrap setrlimit if the new ugetrlimit is not present */ 
+#ifdef L___setrlimit
+#define __NR___setrlimit __NR_setrlimit
+#include <unistd.h>
+#include <sys/resource.h>
+#define RMIN(x, y) ((x) < (y) ? (x) : (y))
+_syscall2(int, __setrlimit, int, resource, const struct rlimit *, rlim);
+int setrlimit (__rlimit_resource_t resource, const struct rlimit *rlimits)
+{
+	struct rlimit rlimits_small;
+	/* We might have to correct the limits values.  Since the old values
+	 * were signed the new values might be too large.  */
+	rlimits_small.rlim_cur = RMIN ((unsigned long int) rlimits->rlim_cur,
+				       RLIM_INFINITY >> 1);
+	rlimits_small.rlim_max = RMIN ((unsigned long int) rlimits->rlim_max,
+				       RLIM_INFINITY >> 1);
+	return(__setrlimit(resource, &rlimits_small));
+}
+#undef RMIN
+#endif
+#else /* We don't need to wrap setrlimit */
 #ifdef L_setrlimit
 #include <unistd.h>
 #include <sys/resource.h>
 _syscall2(int, setrlimit, int, resource, const struct rlimit *, rlim);
 #endif
+#endif /* __NR_setrlimit */
 
 //#define __NR_getrlimit        76
-#ifdef L_getrlimit
+#ifdef L___getrlimit
+/* Only include the old getrlimit if the new one (ugetrlimit) is not around */ 
+#ifndef __NR_ugetrlimit
+#define __NR___getrlimit __NR_getrlimit
 #include <unistd.h>
 #include <sys/resource.h>
-_syscall2(int, getrlimit, int, resource, struct rlimit *, rlim);
+_syscall2(int, __getrlimit, int, resource, struct rlimit *, rlim);
+int getrlimit (__rlimit_resource_t resource, struct rlimit *rlimits)
+{
+	int result;
+	result = __getrlimit(resource, rlimits);
+
+	if (result == -1)
+		return result;
+
+	/* We might have to correct the limits values.  Since the old values
+	 * were signed the infinity value is too small.  */
+	if (rlimits->rlim_cur == RLIM_INFINITY >> 1)
+		rlimits->rlim_cur = RLIM_INFINITY;
+	if (rlimits->rlim_max == RLIM_INFINITY >> 1)
+		rlimits->rlim_max = RLIM_INFINITY;
+	return result;
+}
 #endif
+#endif /* __NR_getrlimit */
 
 //#define __NR_getrusage        77
 #ifdef L_getrusage
@@ -1404,38 +1447,28 @@ _syscall4(ssize_t,sendfile, int, out_fd, int, in_fd, __off_t *, offset, size_t,
 //See sysdeps/linux/<arch>vfork.[cS] for architecture specific implementation...
 
 //#define __NR_ugetrlimit		191	/* SuS compliant getrlimit */
+#ifdef L___ugetrlimit
+#ifdef __NR_ugetrlimit
+#define __NR___ugetrlimit __NR_ugetrlimit
+#include <unistd.h>
+#include <sys/resource.h>
+_syscall2(int, __ugetrlimit, enum __rlimit_resource, resource, struct rlimit *, rlim);
+int getrlimit (__rlimit_resource_t resource, struct rlimit *rlimits)
+{
+	return(__ugetrlimit(resource, rlimits));
+}
+#endif /* __NR_ugetrlimit */
+#endif
+
+
 //#define __NR_mmap2		192
 
 
 //#define __NR_truncate64         193
-#ifdef L_truncate64
-#ifdef __UCLIBC_HAVE_LFS__
-#include <bits/wordsize.h>
-/* Using _syscall2 to pass 64-bit arguments generally only works on 64-bit 
- * systems, so we only implement truncate64/ftruncate64 in that case.  Ports 
- * for processors with shorter word-lengths should define their own custom 
- * versions instead.  */
-#if __WORDSIZE >= 64
-#include <unistd.h>
-_syscall2(int, truncate64, const char *, path, __off64_t, length);
-#endif /* __WORDSIZE >= 64 */
-#endif /* __UCLIBC_HAVE_LFS__ */
-#endif
+//See libc/sysdeps/linux/common/truncate64.c
 
 //#define __NR_ftruncate64        194
-#ifdef L_ftruncate64
-#ifdef __UCLIBC_HAVE_LFS__
-#include <bits/wordsize.h>
-/* Using _syscall2 to pass 64-bit arguments generally only works on 64-bit 
- * systems, so we only implement truncate64/ftruncate64 in that case.  Ports 
- * for processors with shorter word-lengths should define their own custom 
- * versions instead.  */
-#if __WORDSIZE >= 64
-#include <unistd.h>
-_syscall2(int, ftruncate64, int, fd, __off64_t, length);
-#endif /* __WORDSIZE >= 64 */
-#endif /* __UCLIBC_HAVE_LFS__ */
-#endif
+//See libc/sysdeps/linux/common/ftruncate64.c
 
 
 //#define __NR_stat64             195
